function Anew = eig_aware_sparsification(A)
%Vagelis Papalexakis, 2014
%School of Computer Science, Carnegie Mellon University
%Part of the Sparse System Identification Algorithm. 
%Given a square matrix A, we calculate a sparse version of it
%while keeping its eigenvalues within \epsilon (elementwise)

tol = 10^-3;
Anew = A;
lambda_old = eig(A);
isComplex = imag(lambda_old)~=0;

for i = 1:nnz(A)
    Atmp = Anew; Atmp(Atmp==0) = Inf;
    [idxi, idxj] = find(min(min(abs(Atmp)))==abs(Atmp));
    
    Aold = Anew;
    Anew(idxi,idxj) = 0;
    
    lambda_new=eig(Anew);
    
    delta_real = abs(real(lambda_new) - real(lambda_old) );
    delta_imag = abs(imag(lambda_new) - imag(lambda_old) );
    lambda_old = lambda_new;
    
    changeInComplexity = double(isComplex) - double(imag(lambda_new)~=0);
    if sum(changeInComplexity)~=0 || sum((delta_real)>tol)>0 || sum((delta_imag)>tol)>0
       Anew = Aold;
       break
    end
end

disp(sprintf('Successfully deleted %d non-zero connections',i-1))
